// post calculus interpreter script
// aquired 051106-2158, last change 060223-1058

var
  input, newInpt, newRule, inptPtr, rulePtr, levlPtr, varCnt, rex,
  interId, verbosity, stepMode, freshOutput;

var
  rulePool   = { left: [], right: [] },
  resultPool = new Array,
  varLen     = new Array,
  varVal     = new Array;

var
  fillStr    = "  ";

function setVarLen() {
  if (newRule) {
    var match_res;
    if ((match_res = rulePool.left[rulePtr].match(/\[.\]/g)) === null) {
       varCnt = 1
       varLen[0] = 0;
    }
    else {
      varCnt = match_res.length;
      for (var i = 0; i < varCnt; varLen[i++] = input.length);
    }
    varLen.length = varCnt;
    newRule = false;
  }
  else {
    // if only one var exists length experiments make no sense
    if (varCnt == 1) {
      slurpNext();
      return;
    }
    // cut unused varLen, last length needs not to be cut
    for (var i = 0; i < varCnt - 1; i++) {
      if (varVal[i].length < varLen[i])
        varLen[i] = varVal[i].length;
    }
    for (var i = varCnt - 2; i >= 0; i--) {
      varLen[i]--;
      if (varLen[0] < 0) {
        slurpNext();
        break;
      }
      if (varLen[i] < 0)
        varLen[i] = input.length;
      else {
        verbosity == 2 && printDiv(varLen[0] + " | "  + varLen[1] + " | " + varLen[2]);
        break;
      }
    }
  }
}

function setVarVal() {
  var rexStr = rulePool.left[rulePtr], varStr;
  for (var i = 0; i < varCnt; i++) {
    varStr = varLen[i] == 0 ? "()" : "(.{0," + varLen[i] + "})";
    rexStr = rexStr.replace(/\[.\]/, varStr);
  }
  var result = new RegExp("^" + rexStr + "$").exec(input);
  if (result === null) {
    if (isUnbounded()) slurpNext();
    return false;
  }
  for (var i = 0; i < result.length; i++)
    varVal[i] = result[i + 1];
  return true;
}

function isUnbounded() {
  for (var i = 0; i < varLen.length - 1; i++)
    if (varLen[i] < input.length)
      return false;
  return true;
}

function slurpNext() {
   this["new" + (++rulePtr == rulePool.left.length ? "Inpt" : "Rule")] = true;
}

function saveResult() {
  var result = rulePool.right[rulePtr], tmp;
  if (resultPool[levlPtr] === undefined)
    resultPool[levlPtr] = new Array;
  while ((tmp = result.match(/\[.\]/)) !== null)
    result = result.replace(/\[.\]/, varVal[tmp[0].charCodeAt(1) - 65]);
  addUniqueResult(result);
}

function addUniqueResult(result) {
  for (var i = 0; i < resultPool[levlPtr].length; i++)
    if (resultPool[levlPtr][i] == result)
      return;
  resultPool[levlPtr][resultPool[levlPtr].length] = result;
}

function fillCh(num) {
  return fillStr.substr((num = num.toString()).length) + num;
}

function dumpValues() {
  var ruleStr = (rulePool.left[rulePtr] + "->" + rulePool.right[rulePtr]).replace(/\\/g, '');
  printDiv("rule " + fillCh(rulePtr) + " = '" + ruleStr + "'", "values");
  printDiv("input  = '" + input + "'", "_append_nl");
  if (varVal[0] === undefined)
    printDiv("no variables", "vars");
  else {
    printDiv("variables:", "vars");
    printDiv("", "_append_nl")
    for (var i = 0; i < varCnt; i++)
      printDiv("[" + String.fromCharCode(i + 65) + "] = '" + varVal[i] + "' ", "_append");
  }
}

function dumpResult() {
  if (verbosity) {
    printDiv("results:", "result");
    for (var i = 0; i < levlPtr; i++) {
      printDiv("level " + fillCh(i) + " = ", "_append_nl");
      for (var k = 0; k < resultPool[i].length; k++)
        printDiv("'" + resultPool[i][k] + "' ", "_append");
    }
  }
  else {
    var ptr = levlPtr - 1;
    printDiv("level " + fillCh(ptr) + " = ", ptr ? "_append_nl" : "result");
    for (var k = 0; k < resultPool[ptr].length; k++)
      printDiv("'" + resultPool[ptr][k] + "' ", "_append");
  }
}

function getRoot() {
  return document.getElementById("output");
}

function printDiv(str, classId) {
  switch (classId) {
    case "_append_nl":
      with (getRoot().lastChild) {
        appendChild(document.createElement("br"));
        appendChild(document.createTextNode(str));
      }
      break;
    case "_append":
      getRoot().lastChild.lastChild.appendData(str);
      break;
    default:
      var div = document.createElement("div");
      div.setAttribute("class", classId);
      div.appendChild(document.createTextNode(str));
      getRoot().appendChild(div);
  }
}

function resetOutput() {
  getRoot().parentNode.replaceChild(freshOutput.cloneNode(false), getRoot());
}

function cloneOutput() {
  freshOutput = getRoot().cloneNode(false);
}

function interPause() {
  with (document.frm)
    fCont.disabled = ! (fPause.disabled = true);
  window.clearInterval(interId);
}

function interCont() {
  with (document.frm)
    fPause.disabled = ! (fCont.disabled  = true);
  interId = window.setInterval('interLoop()', 1);
}

function interLoop() {
  input   = resultPool[levlPtr - 1][inptPtr];
  newRule = true;
  newInpt = false;
  rulePtr = 0;

  while (true) {
    setVarLen();
    if (newInpt) break;
    if (newRule) continue;
    if (setVarVal()) verbosity && dumpValues(), saveResult();
    if (newInpt) break;
  }
  if (++inptPtr == resultPool[levlPtr - 1].length) {
    inptPtr = 0;
    levlPtr++;
    if (levlPtr - 1 == resultPool.length) {
      printDiv("computation done", "done");
      document.frm.fPause.disabled = true;
      window.clearInterval(interId);
      return;
    }
    dumpResult();
    stepMode  && interPause();
  }
}

function initRules() {
  var rules = document.frm.fRules.value.concat("\n").match(/.*->.*\r*\n/g);
  if (! rules) return false;
  with (rulePool)
    left.length = right.length = 0;
  for (var i = 0; i < rules.length; i++) {
    rules[i] = rules[i].replace(/\r*\n/, "").split("->");
    rulePool.left[i]  = rules[i][0];
    rulePool.right[i] = rules[i][1];
  }
  for (var elm in rulePool.left)
    rulePool.left[elm] = rulePool.left[elm].replace(/\*|\+|\$|\(|\)|\|/g, "\\$&");
  return true;
}

function main() {
  document.frm.fResetOutput.checked && resetOutput();
  if (initRules()) {
    resultPool.length = levlPtr = inptPtr = 0;
    resultPool[-1] = [document.frm.fInput.value];
    stepMode  = document.frm.fStepMode.checked;
    verbosity = document.frm.fVerbosity.checked;
    interCont();
  }
  else
    printDiv("invalid rule definition", "error");
}
